/*
 * This file is part of TechReborn, licensed under the MIT License (MIT).
 *
 * Copyright (c) 2017 TechReborn
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package techreborn.items;

import net.minecraft.client.Minecraft;
import net.minecraft.client.util.ITooltipFlag;
import net.minecraft.creativetab.CreativeTabs;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.inventory.Container;
import net.minecraft.inventory.IInventory;
import net.minecraft.item.ItemStack;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.NonNullList;
import net.minecraft.world.World;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;
import net.minecraftforge.items.CapabilityItemHandler;
import net.minecraftforge.items.IItemHandler;
import reborncore.api.tile.IUpgrade;
import reborncore.common.powerSystem.TilePowerAcceptor;
import reborncore.common.recipes.RecipeCrafter;
import reborncore.common.tile.IMachineSlotProvider;
import reborncore.common.tile.TileLegacyMachineBase;
import reborncore.common.util.InventoryHelper;
import reborncore.common.util.ItemNBTHelper;
import techreborn.Core;
import techreborn.client.TechRebornCreativeTabMisc;
import techreborn.client.container.builder.BuiltContainer;
import techreborn.client.container.builder.ContainerBuilder;
import techreborn.client.gui.upgrades.GuiSideConfig;
import techreborn.init.ModItems;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import java.security.InvalidParameterException;
import java.util.List;

public class ItemUpgrades extends ItemTR implements IUpgrade {

	public static final String[] types = new String[] { "overclock", "transformer", "energy_storage", "range", "ejection", "injection" };

	public ItemUpgrades() {
		func_77655_b("techreborn.upgrade");
		func_77627_a(true);
		func_77637_a(TechRebornCreativeTabMisc.instance);
		func_77625_d(1);
	}

	public static ItemStack getUpgradeByName(String name, int count) {
		for (int i = 0; i < types.length; i++) {
			if (types[i].equalsIgnoreCase(name)) {
				return new ItemStack(ModItems.UPGRADES, count, i);
			}
		}
		throw new InvalidParameterException("The upgrade " + name + " could not be found.");
	}

	public static ItemStack getUpgradeByName(String name) {
		return getUpgradeByName(name, 1);
	}

	@Override
	// gets Unlocalized Name depending on meta data
	public String func_77667_c(ItemStack itemStack) {
		int meta = itemStack.func_77952_i();
		if (meta < 0 || meta >= types.length) {
			meta = 0;
		}

		return super.func_77658_a() + "." + types[meta];
	}

	// Adds Dusts SubItems To Creative Tab
	@Override
	public void func_150895_a(CreativeTabs creativeTabs, NonNullList list) {
		if(!func_194125_a(creativeTabs)){
			return;
		}
		for (int meta = 0; meta < types.length; ++meta) {
			list.add(new ItemStack(this, 1, meta));
		}
	}

	@Override
	public void func_77624_a(ItemStack stack, World world, List<String> tooltip, ITooltipFlag flag) {
		super.func_77624_a(stack, world, tooltip, flag);
		if (stack.func_77952_i() == 4 || stack.func_77952_i() == 5) {
			tooltip.add("Facing: " + getFacing(stack).func_176610_l());
			String text = Core.proxy.getUpgradeConfigText();
			if (!text.isEmpty()) {
				tooltip.add(text);
			}
		}
	}

	@Override
	public void process(
		@Nonnull
			TileLegacyMachineBase machineBase,
		@Nullable
			RecipeCrafter crafter,
		@Nonnull
			ItemStack stack) {

		if (stack.func_77952_i() == 0) {
			if (crafter != null) {
				crafter.addSpeedMulti(0.25);
				crafter.addPowerMulti(0.5);
			}

		} else if (stack.func_77952_i() == 4) {
			EnumFacing dir = getFacing(stack);
			TileEntity tileEntity = machineBase.func_145831_w().func_175625_s(machineBase.func_174877_v().func_177972_a(dir));
			if (tileEntity instanceof IInventory) {
				if (crafter != null) {
					for (Integer outSlot : crafter.outputSlots) {
						ItemStack outputStack = crafter.inventory.func_70301_a(outSlot);
						if (!outputStack.func_190926_b()) {
							int amount = InventoryHelper.testInventoryInsertion((IInventory) tileEntity, outputStack, dir);
							if (amount > 0) {
								InventoryHelper.insertItemIntoInventory((IInventory) tileEntity, outputStack);
								crafter.inventory.func_70298_a(outSlot, amount);
							}
						}
					}
				} else if (machineBase instanceof IMachineSlotProvider) {
					IMachineSlotProvider slotProvider = (IMachineSlotProvider) machineBase;
					for (Integer outSlot : slotProvider.getOuputSlots()) {
						ItemStack outputStack = slotProvider.getMachineInv().func_70301_a(outSlot);
						if (!outputStack.func_190926_b()) {
							int amount = InventoryHelper.testInventoryInsertion((IInventory) tileEntity, outputStack, dir);
							if (amount > 0) {
								InventoryHelper.insertItemIntoInventory((IInventory) tileEntity, outputStack);
								slotProvider.getMachineInv().func_70298_a(outSlot, amount);
							}
						}
					}
				} else {
					IInventory inventory = machineBase;
					for (int outSlot = 0; outSlot < inventory.func_70302_i_(); outSlot++) {
						ItemStack outputStack = inventory.func_70301_a(outSlot);
						if (!outputStack.func_190926_b()) {
							int amount = InventoryHelper.testInventoryInsertion((IInventory) tileEntity, outputStack, dir);
							if (amount > 0) {
								InventoryHelper.insertItemIntoInventory((IInventory) tileEntity, outputStack);
								inventory.func_70298_a(outSlot, amount);
							}
						}
					}
				}
			}
		} else if (stack.func_77952_i() == 5) {
			if (machineBase.func_145831_w().func_82737_E() % 10 != 0) {
				return;
			}
			EnumFacing dir = getFacing(stack);
			TileEntity tileEntity = machineBase.func_145831_w().func_175625_s(machineBase.func_174877_v().func_177972_a(dir));
			if (tileEntity != null && tileEntity.hasCapability(CapabilityItemHandler.ITEM_HANDLER_CAPABILITY, dir.func_176734_d())) {
				IItemHandler itemHandler = tileEntity.getCapability(CapabilityItemHandler.ITEM_HANDLER_CAPABILITY, dir.func_176734_d());
				for (int i = 0; i < itemHandler.getSlots(); i++) {
					ItemStack stack1 = itemHandler.getStackInSlot(i);
					if (crafter == null || crafter.isStackValidInput(stack1)) {
						ItemStack extractedStack = itemHandler.extractItem(i, 1, true);
						int amount = InventoryHelper.testInventoryInsertion(machineBase, extractedStack, null);
						if (amount > 0) {
							extractedStack = itemHandler.extractItem(i, 1, false);
							extractedStack.func_190920_e(1);
							InventoryHelper.insertItemIntoInventory(machineBase, extractedStack);
						}
					}
				}
			}

		}
		if (machineBase instanceof TilePowerAcceptor) {
			if (stack.func_77952_i() == 2) {
				TilePowerAcceptor acceptor = (TilePowerAcceptor) machineBase;
				acceptor.extraPowerStoage += 40000;
			}
			if (stack.func_77952_i() == 1) {
				TilePowerAcceptor acceptor = (TilePowerAcceptor) machineBase;
				acceptor.extraTeir += 1;
			}
		}
	}

	@Override
	@SideOnly(Side.CLIENT)
	public void handleRightClick(TileEntity tile, ItemStack stack, Container container, int slotID) {
		if (tile.func_145831_w().field_72995_K) {
			if (stack.func_77952_i() == 4 || stack.func_77952_i() == 5) {
				//TODO use the full gui handler
				Minecraft.func_71410_x().func_147108_a(new GuiSideConfig(Minecraft.func_71410_x().field_71439_g, tile, getContainer(Minecraft.func_71410_x().field_71439_g), slotID));
			}
		}
	}

	@SideOnly(Side.CLIENT)
	public BuiltContainer getContainer(EntityPlayer player) {
		return new ContainerBuilder("sides").create();
	}

	public EnumFacing getFacing(ItemStack stack) {
		return EnumFacing.field_82609_l[ItemNBTHelper.getInt(stack, "side", 0)];
	}
}
